@@ -1,10 +1,8 @@
 #coding:utf-8
 
-import json
-import logging
 import pika
 import functools
-from pika.exchange_type import ExchangeType
+from pika.exceptions import StreamLostError, ConnectionWrongStateError
 
 
 class Subscriber(object):
@@ -79,6 +77,10 @@ class Subscriber(object):
                 auto_ack=self._get_params("_auto_ack", "queue")
                 )
 
+    def reconnect(self):
+        self.connect()
+        print('reconnect to %s', self._url)
+
 
     def on_message(self, chan, method_frame, header_frame, body, userdata=None):
         print('Delivery properties: %s, message metadata: %s', method_frame, header_frame)
@@ -87,13 +89,23 @@ class Subscriber(object):
 
 
     def start(self):
-        try:
-            self._channel.start_consuming()
-        except KeyboardInterrupt:
-            self._channel.stop_consuming()
+        while True:
+            try:
+                self._channel.start_consuming()
+            except KeyboardInterrupt:
+                self._channel.stop_consuming()
+                break
+            except (StreamLostError,
+                    ConnectionWrongStateError,
+                    ConnectionResetError):
+                self.reconnect()
+            except:
+                raise e
+
 
     def __del__(self):
-        self._connection.close()
+        if self._connection is not None:
+            self._connection.close()
 
 
 if __name__=='__main__':
